--
-- Agribumper
-- Specialization for installation of the Agribumper
--
-- @author John Deere 6930
-- @date 27/10/2019
-- @version 2.0
--

Agribumper = {};
Agribumper.modDir = g_currentModDirectory;

function Agribumper.prerequisitesPresent(specializations)	
	return SpecializationUtil.hasSpecialization(Drivable, specializations) and SpecializationUtil.hasSpecialization(Lights, specializations) and SpecializationUtil.hasSpecialization(AttacherJoints, specializations);
end;

function Agribumper.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", Agribumper);
end;

function Agribumper.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "onLoadAgribumperFromXMLFile", Agribumper.onLoadAgribumperFromXMLFile);
end;

function Agribumper:onLoad(savegame)	
	self.spec_agribumper = {};
	local spec = self.spec_agribumper;
	
	local agribumperId = self.configurations["agribumper"];		
	spec.agribumper = {}
	
	local agribumperKey = nil;
	if hasXMLProperty(self.xmlFile, "vehicle.agribumper") then
		agribumperKey = "vehicle.agribumper";
	end;
	
	if agribumperKey ~= nil then
		--load from vehicle.xml
		spec.agribumper = self:onLoadAgribumperFromXMLFile(self.xmlFile, agribumperKey, agribumperId, self);
		ObjectChangeUtil.updateObjectChanges(self.xmlFile, agribumperKey..".agribumperConfigurations.agribumperConfiguration", agribumperId, self.components, self);
	else
		--new load routine from config.xml
		if g_agribumperData.configData[self.configFileName] ~= nil then
			local configData = g_agribumperData.configData[self.configFileName];
			local xmlFile = loadXMLFile("agribumperConfigXML", g_agribumperData.configXMLFilename);	
			if xmlFile ~= nil and xmlFile ~= 0 then
				local baseKey = string.format("xmlConfigData.vehicles.vehicle(%d)", configData.xmlId);
				if hasXMLProperty(xmlFile, baseKey) then
					spec.agribumper = self:onLoadAgribumperFromXMLFile(xmlFile, baseKey, agribumperId, self);
					ObjectChangeUtil.updateObjectChanges(xmlFile, baseKey..".agribumperConfigurations.agribumperConfiguration", agribumperId, self.components, self);
				end;
				delete(xmlFile);
			end;
		end;
	end;
end;

function Agribumper:onLoadAgribumperFromXMLFile(xmlFile, baseKey, agribumperId, object)
	local agribumper = {};
	local node = I3DUtil.indexToObject(object.components, getXMLString(xmlFile, baseKey..".params#node"), object.i3dMappings)
	if node ~= nil then
		local agribumperXmlFilename = getXMLString(xmlFile, baseKey..".params#filename");
		if agribumperXmlFilename ~= nil then
			agribumperXmlFilename = Utils.getFilename(agribumperXmlFilename, Agribumper.modDir)
			local agribumperXmlFile = loadXMLFile("agribumperXmlFilename", agribumperXmlFilename)
			
			if agribumperXmlFile ~= nil and agribumperXmlFile ~= 0 then
				local i3dFilename = getXMLString(agribumperXmlFile, "agribumper.filename")
				if i3dFilename ~= nil then
					local i3dNode = g_i3DManager:loadSharedI3DFile(i3dFilename, Agribumper.modDir, false, false, false)
					if i3dNode ~= nil and i3dNode ~= 0 then
						local rootNode = I3DUtil.indexToObject(i3dNode, getXMLString(agribumperXmlFile, "agribumper.rootNode#node"))
						if rootNode ~= nil then
							--link to vehiclenode
							link(node, rootNode);
							
							--i3dMapping
							if object.i3dMappings ~= nil then
								local index = getChildIndex(rootNode);
								local parentIndex = object.i3dMappings[getXMLString(xmlFile, baseKey..".params#node")];								
								local j = 0;
								while true do
									local key = string.format("agribumper.i3dMappings.i3dMapping(%d)", j)
									if not hasXMLProperty(agribumperXmlFile, key) then
										break;
									end;
									local id = getXMLString(agribumperXmlFile, key.."#id");
									local nodeIndex = getXMLString(agribumperXmlFile, key.."#node");
									if object.i3dMappings[id] == nil and nodeIndex ~= nil and id ~= nil then
										local idIndex = parentIndex.."|"..nodeIndex;
										idIndex = string.format(idIndex, index);
										object.i3dMappings[id] = idIndex;
									else
										g_logManager:xmlError(agribumperXmlFilename, "i3d-Mapping[%s] already exists!", id);
									end;												
									j = j + 1;
								end;
							else
								g_logManager:xmlError(object.configFileName, "i3dMapping doesn't exists");
							end;
							
							--position and rotation
							local x, y, z = StringUtil.getVectorFromString(getXMLString(xmlFile,  baseKey..".params#translation"))
							setTranslation(rootNode, x, y, z);
							local rot = StringUtil.getRadiansFromString(Utils.getNoNil(getXMLString(xmlFile, baseKey..".params#rotation"), "0 0 0"), 3)
							setRotation(rootNode, unpack(rot));
							agribumper.filename = i3dFilename;
							agribumper.rootNode = rootNode;
							
							--shader application							
							local shaderParameterName = getXMLString(xmlFile, baseKey..".params#shaderParameterName");
							agribumper.shaderParameterName = shaderParameterName;							
							local color = ConfigurationUtil.getColorFromString(getXMLString(xmlFile, baseKey..".params#color"));
							if color == nil then
								color = ConfigurationUtil.getColorFromString(getXMLString(xmlFile, baseKey..".params#shaderParameter"));
							end;
							local materialId = getXMLInt(xmlFile, baseKey..".params#materialId");
							local useBaseMaterial = Utils.getNoNil(getXMLString(xmlFile, baseKey..".params#useBaseMaterial"), false);
							local useDesignMaterial = Utils.getNoNil(getXMLString(xmlFile, baseKey..".params#useDesignMaterial"), false);
							if color == nil then
								if useBaseMaterial then
									if object.configurations["baseMaterial"] ~= nil then
										local baseMaterialId = object.configurations["baseMaterial"];
										color = ConfigurationUtil.getColorByConfigId(object, "baseMaterial", baseMaterialId);
									else
										g_logManager:xmlError(object.configFileName, "Can't load agribumper material data, configuration 'baseMaterial' does not exist.");												
									end;
								elseif useDesignMaterial then
								
									if object.configurations["designMaterial"] ~= nil then									
										local designMaterialId = object.configurations["designMaterial"];
										color = ConfigurationUtil.getColorByConfigId(object, "designMaterial", designMaterialId);
									else
										g_logManager:xmlError(object.configFileName, "Can't load agribumper material data, configuration 'designMaterialId' does not exist.");												
									end;
								else
									g_logManager:xmlError(object.configFileName, "Can't load agribumper material data, selected material is not valid.");
								end;
							end;							
							if shaderParameterName ~= nil and color ~= nil then
								if getHasClassId(rootNode, ClassIds.SHAPE) then
									if materialId == nil then
										_,_,_,materialId = getShaderParameter(rootNode, shaderParameterName)
									end
									local r, g, b = unpack(color);
									if Utils.getNoNil(getXMLBool(xmlFile, baseKey .. "#recursive"), false) then
										I3DUtil.setShaderParameterRec(rootNode, shaderParameterName, r, g, b, materialId);
									else
										setShaderParameter(rootNode, shaderParameterName, r, g, b, materialId, false);
									end;
								else
									print("Warning: Could not set agribumper color to '"..getName(rootNode).."' because node is not a shape!");
								end;
							end;
							
							--sharedLights
							local spec_lights = object.spec_lights;		
							if spec_lights ~= nil and spec_lights.sharedLights ~= nil then
								local j = 0;
								while true do
									local key = string.format("agribumper.lights.sharedLight(%d)", j)
									if not hasXMLProperty(agribumperXmlFile, key) then
										break;
									end;
									local sharedLight = {};
									if object:loadSharedLight(agribumperXmlFile, key, sharedLight) then
										table.insert(spec_lights.sharedLights, sharedLight)
									end
									j = j + 1;
								end;
							else
								g_logManager:xmlError(object.configFileName, "Can't load agribumper shared lights, no lights avaiable in mod.");
							end;
							
							--startRotation
							if agribumperId ~= 1 then
								local spec_attacherJoints = object.spec_attacherJoints;
								local startRot = StringUtil.getRadiansFromString(getXMLString(xmlFile, baseKey..".attacherJoint#startRotation"), 3);
								local index =  Utils.getNoNil(getXMLInt(xmlFile, baseKey..".attacherJoint#index"), 1);
								local attacherJoint = spec_attacherJoints.attacherJoints[index];
								if attacherJoint ~= nil and attacherJoint.bottomArm ~= nil and startRot ~= nil then											
									attacherJoint.bottomArm.rotX, attacherJoint.bottomArm.rotY, attacherJoint.bottomArm.rotZ = startRot[1],startRot[2],startRot[3];
									setRotation(attacherJoint.bottomArm.rotationNode, attacherJoint.bottomArm.rotX, attacherJoint.bottomArm.rotY, attacherJoint.bottomArm.rotZ)
									if object.setMovingToolDirty ~= nil then
										object:setMovingToolDirty(attacherJoint.bottomArm.rotationNode)
									end
								end;
							end;
						end;
						delete(i3dNode);
					end;
				end;
				delete(agribumperXmlFile);
			end;
		end;
	end;
	return agribumper;
end;