--
-- AgribumperSourceFile
-- Basic for Agribumper
--
-- @author John Deere 6930
-- @date 23/10/2019
-- @version 2.0
--

AgribumperSourceFile = {}; 
AgribumperSourceFile.modDir = g_currentModDirectory;
AgribumperSourceFile.configXMLFilename = Utils.getFilename("agribumperConfigFile.xml", AgribumperSourceFile.modDir);
AgribumperSourceFile.configData = nil;

if getfenv(0)["g_agribumperData"] == nil then
	getfenv(0)["g_agribumperData"] = AgribumperSourceFile;
end;

if g_configurationManager.configurationNameToInt["agribumper"] == nil then
	g_configurationManager:addConfigurationType("agribumper", g_i18n:getText("configuration_agribumper"), nil, nil, nil, ConfigurationManager.SELECTOR_MULTIOPTION);
end;

if AgribumperSourceFile.configData == nil then
	--new configData Loading
	
	AgribumperSourceFile.configData = {};
	
	local xmlFile = loadXMLFile("agribumperConfigXML", AgribumperSourceFile.configXMLFilename);	
	if xmlFile ~= nil and xmlFile ~= 0 then
		local i=0;
		while true do
			local baseName = string.format("xmlConfigData.vehicles.vehicle(%d)", i);
			if not hasXMLProperty(xmlFile, baseName) then
				break;
			end;
			
			local vehicleFilename = getXMLString(xmlFile, baseName .. "#vehicleFilename");
			local vehicleFilenames = nil;
			if StringUtil.startsWith(vehicleFilename, "$moddir$") then
				local splitStrings = StringUtil.splitString("dir$", vehicleFilename)
				vehicleFilename = g_modsDirectory..splitStrings[2];
				
			elseif StringUtil.startsWith(vehicleFilename, "$pdlc$") then
				local splitStrings = StringUtil.splitString("dlc$", vehicleFilename)
				vehicleFilenames = {};
				for k, v in pairs(g_dlcsDirectories) do
					local filename = v.path..splitStrings[2];
					table.insert(vehicleFilenames, filename);
				end;
			end;
			
			local agribumperConfigurations = {};			
			local numConfigs = Utils.getNoNil(getXMLInt(xmlFile, baseName..".agribumperConfigurations#numConfigs"), 0);
			for p = 0, numConfigs-1 do
				local key3 = string.format(baseName..".agribumperConfigurations.agribumperConfiguration(%d)", p);
				if not hasXMLProperty(xmlFile, key3) then
					break;
				end;
				
				local entry = {};
				entry.name = g_i18n:getText(getXMLString(xmlFile, key3.."#name"));
				entry.price = Utils.getNoNil(getXMLFloat(xmlFile, key3.."#price"), 0.0);
				entry.dailyUpkeep = Utils.getNoNil(getXMLInt(xmlFile, key3.."#dailyUpkeep"), 0);	
				agribumperConfigurations[p+1] = entry;
				p = p+1;
			end;
			
			if vehicleFilename ~= nil and vehicleFilename ~= "" then
				if type(vehicleFilenames) == "table" then
					for k,v in pairs(vehicleFilenames) do
						AgribumperSourceFile.configData[v] = {};
						AgribumperSourceFile.configData[v].xmlId = i;
						if table.getn(agribumperConfigurations) > 0 then
							AgribumperSourceFile.configData[v].agribumperConfigurations = agribumperConfigurations;
						end;
					end;
				else
					AgribumperSourceFile.configData[vehicleFilename] = {};
					AgribumperSourceFile.configData[vehicleFilename].xmlId = i;
					if table.getn(agribumperConfigurations) > 0 then
						AgribumperSourceFile.configData[vehicleFilename].agribumperConfigurations = agribumperConfigurations;
					end;
				end;
			end;
			i = i + 1;
		end;
		delete(xmlFile);
	end;
end;

function addAgribumperConfiguration(xmlFile, superFunc, baseXMLName, baseDir, customEnvironment, isMod, storeItem)
	local configurations = superFunc(xmlFile, baseXMLName, baseDir, customEnvironment, isMod, storeItem)
	
	local agribumperKey = nil;
	if hasXMLProperty(xmlFile, "vehicle.agribumper") then
		agribumperKey = "vehicle.agribumper";
	end;
	
	if agribumperKey ~= nil then
		--load from vehicle.xml
		configurations["agribumper"] = {};
		local numConfigs = Utils.getNoNil(getXMLInt(xmlFile, agribumperKey..".agribumperConfigurations#numConfigs"), 0);
		for p = 0, numConfigs-1 do
			local key3 = string.format(agribumperKey..".agribumperConfigurations.agribumperConfiguration(%d)", p);
			if not hasXMLProperty(xmlFile, key3) then
				break;
			end;			
			
			local name = g_i18n:getText(getXMLString(xmlFile, key3.."#name"));
			local price = Utils.getNoNil(getXMLFloat(xmlFile, key3.."#price"), 0.0);
			local dailyUpkeep = Utils.getNoNil(getXMLInt(xmlFile, key3.."#dailyUpkeep"), 0);					
			local configurationItem = StoreItemUtil.addConfigurationItem(configurations["agribumper"], name, "agribumper", price, dailyUpkeep, (p == 0));
			p = p+1;
		end;
	else
		--new better load from config.xml		
		if AgribumperSourceFile.configData[storeItem.xmlFilename] ~= nil then		
			configurations["agribumper"] = {};			
			for _,config in pairs(AgribumperSourceFile.configData[storeItem.xmlFilename].agribumperConfigurations) do
				local configurationItem = StoreItemUtil.addConfigurationItem(configurations["agribumper"], config.name, "agribumper", config.price, config.dailyUpkeep, (p == 0));
			end;
		end;
	end;
	return configurations;
end;

StoreItemUtil.getConfigurationsFromXML = Utils.overwrittenFunction(StoreItemUtil.getConfigurationsFromXML, addAgribumperConfiguration);